*************************************************************************************
*														*
*	*.BMP file loader V1.00. This version only supports the displaying of 16	*
*	colour bitmaps. This is an include file for invader1000.x68				*
*														*
*************************************************************************************
*
* this is the name of the 16 colour bitmap file to use as the background

name
	dc.b	'invader.bmp',0			* background file name
	ds.w	0					* ensure even

image_x	EQU	200				* x offset of image in display window

*************************************************************************************
*
* opens and loads an image

abort_load
	MOVEQ		#50,d0			* else close all files
	TRAP		#15

abort_display
	RTS

load_image
	LEA		name(pc),a1			* set the file name pointer
	MOVE		#51,d0			* open an existing file
	TRAP		#15

	TST.w		d0				* did the file open
	BNE.s		abort_load			* if not then abort the file load

	LEA		im_file(a3),a1		* point to the file buffer
	MOVE.l	#$FF0000,d2			* set the byte count to load everything
	MOVEQ		#53,d0			* read from the file
	TRAP		#15

	MOVEQ		#56,d0			* close the file
	TRAP		#15

	MOVE.w	(a1)+,d0			* get the signature word
	CMPI.w	#'BM',d0			* compare the signature word with "BM"
	BNE.s		abort_display		* if incorrect abort the file display

	LEA		$0C(a1),a1			* skip the rest of the file header

	MOVEQ		#$28,d2			* set the size that is expected
	BSR		get_long			* get the image header size
	CMP.l		d2,d0				* compare them
	BNE.s		abort_display		* if incorrect abort the file display

	BSR		get_long			* get the image width
	SUBQ.l	#1,d0				* make the image width 0 to n
	MOVE.l	d0,im_width(a3)		* save the image width
	BSR		get_long			* get the image height
	MOVE.l	d0,im_height(a3)		* save the image height

	BSR.s		get_word			* get the number of planes
	CMPI.w	#1,d0				* compare with expected number
	BNE.s		abort_display		* if incorrect abort the file display

	BSR.s		get_word			* get the image depth
	CMPI.w	#4,d0				* compare the image depth with 16 colour
	BNE.s		abort_display		* if incorrect abort the file display

	MOVE.w	d0,im_depth(a3)		* save the image depth

	BSR.s		get_long			* get the compression type
	TST.l		d0				* test for uncompressed file
	BNE.s		abort_display		* if incorrect abort the file display

	LEA		$14(a1),a1			* skip the rest of the header

	BSR.s		load_palette		* load the palette

	MOVE.l	a1,-(sp)			* save the file pointer as the image needs to
							* be rendered in both screem buffer pages
	BSR.s		display_image		* render the image to the screen

	MOVEQ		#94,d0			* copy screen buffer to main
	TRAP		#15

	MOVEA.l	(sp)+,a1			* restore the file pointer and ...
*##	BRA.s		display_image		* render the image to the screen


*************************************************************************************
*
* display the image

display_image
	MOVE.w	#$FF00,d1			* clear screen
	MOVEQ		#11,d0			* position cursor
	TRAP		#15

	MOVE.l	im_height(a3),d7		* get the image height
	SUBQ.l	#1,d7				* adjust for the loop type
	MOVE.w	d7,d5				* set the end y co-ordinate

	MOVE.l	im_width(a3),d6		* get the image width
	ADDQ.l	#1,d6				* make 1 to n
	NEG.l		d6				* make -ve
	MOVEQ		#6,d3				* set the byte remainder mask
	AND.l		d6,d3				* mask the byte remainder
	ASR.w		#1,d3				* / 2 pixels per byte
bpp4_y_loop
	MOVE.l	d7,-(sp)			* save the y count

	MOVE.l	im_width(a3),d6		* get the image width
	MOVE.w	#image_x,d4			* set the image x origin
bbp4_byte_loop
	MOVE.b	(a1)+,d7			* get a pixel word
	BSR		plot_pixel			* plot the pixel

	SUBQ.l	#1,d6				* decrement the pixel count
	BMI.s		bbp4_next_line		* exit the pixel loop if done

	BSR		plot_pixel			* plot the pixel

	DBF		d6,bbp4_byte_loop		* loop for next line

bbp4_next_line
	ADDA.w	d3,a1				* skip the remainder bytes
	SUBQ.w	#1,d5				* decrement the y co-ordinate
	MOVE.l	(sp)+,d7			* restore the y count
	DBF		d7,bpp4_y_loop		* loop for next line

	RTS


*************************************************************************************
*
* get a little endian word, this could be at an even or an odd address

get_word
	MOVE.b	(a1)+,d0			* get the low byte
	ROL.w		#8,d0				* move it
	MOVE.b	(a1)+,d0			* get the high byte
	ROL.w		#8,d0				* swap the bytes
	RTS


*************************************************************************************
*
* get a little endian longword, this could be at an even or an odd address

get_long
	BSR.s		get_word			* get a little endian word
	SWAP		d0				* move it
	BSR.s		get_word			* get a little endian word
	SWAP		d0				* swap the words
	RTS


*************************************************************************************
*
* load the palette. this routine assumes a 16 colour palette.

load_palette
	LEA		im_palette(a3),a2		* point to the palette
	MOVEQ		#$0F,d7			* set the number of colours - 1

* get the bitmap palette colour values. palette colour values are 32 bit values stored
* as B,G,R,$00. EASy68K wants 32 bit colour values in the form $00,B,G,R

load_palette_loop
	MOVEQ		#0,d0				* clear the longword
	MOVE.b	(a1)+,d0			* get the blue byte
	SWAP		d0				* move it to the high word
	MOVE.b	(a1)+,d0			* get the green byte
	ROL.w		#8,d0				* move it to the high byte
	MOVE.b	(a1)+,d0			* get the red byte
	ADDQ.w	#1,a1				* skip the zero byte

	MOVE.l	d0,(a2)+			* save the value to the palette
	DBF		d7,load_palette_loop	* loop for the next colour

	RTS


*************************************************************************************
*
* get the colour and plot a pixel

plot_pixel
	ROL.b		#4,d7				* shift the pixels
	MOVEQ		#$0F,d1			* set the pixel mask
	AND.w		d7,d1				* get the pixel

	ASL.w		#2,d1				* 4 bytes per longword colour
	MOVE.l	im_palette(a3,d1.w),d1	* get the colour

	MOVEQ		#80,d0			* set the pen colour
	TRAP		#15

	MOVE.w	d4,d1				* copy the x co-ordinate
	MOVE.w	d5,d2				* copy the y co-ordinate

	MOVEQ		#82,d0			* draw a pixel
	TRAP		#15

	ADDQ.w	#1,d4				* increment the x co-ordinate
	RTS


*************************************************************************************